@@ -1168,14 +1168,26 @@ compute_diff(const type_base_sptr,
 	     const type_base_sptr,
 	     diff_context_sptr ctxt);
 
+/// Convert the type of a particular diff node into the generic @ref
+/// diff_sptr type.
+///
+/// @tparam DiffNodePtr the specific type of the diff node to convert
+/// into @ref diff_sptr
+///
+/// @param n the diff node to consider.
+///
+/// @return the instance of @ref diff_sptr @p n was converted into.
+template <class DiffNodePtr>
+diff_sptr
+is_diff(DiffNodePtr& n)
+{return std::dynamic_pointer_cast<diff>(n);}
+
 /// The base class of diff between types.
 class type_diff_base : public diff
 {
   struct priv;
   std::unique_ptr<priv> priv_;
 
-  type_diff_base();
-
 protected:
   type_diff_base(type_base_sptr	first_subject,
 		 type_base_sptr	second_subject,
@@ -1183,6 +1195,8 @@ protected:
 
 public:
 
+  type_diff_base() = delete;
+
   virtual enum change_kind
   has_local_changes() const = 0;
 
@@ -1440,7 +1454,7 @@ class ptr_to_mbr_diff : public type_diff_base
   struct priv;
   std::unique_ptr<priv> priv_;
 
-  ptr_to_mbr_diff()  = default;
+  ptr_to_mbr_diff()  = delete;
 
 protected:
   ptr_to_mbr_diff(const ptr_to_mbr_type_sptr& first,
@@ -2798,16 +2812,24 @@ public:
   size_t num_leaf_func_changes() const;
   void num_leaf_func_changes(size_t);
 
+  size_t num_leaf_func_with_incompatible_changes() const;
+  void num_leaf_func_with_incompatible_changes(size_t);
+
   size_t num_leaf_func_changes_filtered_out() const;
   void num_leaf_func_changes_filtered_out(size_t);
   size_t net_num_leaf_func_changes() const;
+  size_t net_num_leaf_func_non_incompatible_changes() const;
 
   size_t num_leaf_var_changes() const;
   void num_leaf_var_changes(size_t);
 
+  size_t num_leaf_var_with_incompatible_changes() const;
+  void num_leaf_var_with_incompatible_changes(size_t);
+
   size_t num_leaf_var_changes_filtered_out() const;
   void num_leaf_var_changes_filtered_out(size_t);
   size_t net_num_leaf_var_changes() const;
+  size_t net_num_leaf_var_non_incompatible_changes() const;
 
   size_t num_added_unreachable_types() const;
   void num_added_unreachable_types(size_t);
@@ -2988,7 +3010,10 @@ void
 clear_redundancy_categorization(corpus_diff_sptr diff_tree);
 
 void
-apply_filters(corpus_diff_sptr diff_tree);
+apply_filters_and_categorize_diff_node_tree(diff_sptr& diff_tree);
+
+void
+apply_filters_and_categorize_diff_node_tree(corpus_diff_sptr& c);
 
 bool
 is_diff_of_variadic_parameter_type(const diff*);
