@@ -261,7 +261,7 @@ public:
   /// that ABI artifacts that are to be compared all need to be
   /// created within the same environment.
   reader(const string&		elf_path,
-	 const vector<char**>&	debug_info_root_paths,
+	 const vector<string>&	debug_info_root_paths,
 	 environment&		env)
     : elf_based_reader(elf_path, debug_info_root_paths, env),
       ctfa(), ctf_sect(), symtab_sect(), strtab_sect()
@@ -311,13 +311,13 @@ public:
   ///
   void
   initialize(const string&		elf_path,
-             const vector<char**>&	debug_info_root_paths,
+             const vector<string>&	debug_info_root_paths,
              bool			load_all_types = false,
              bool			linux_kernel_mode = false)
   {
     reset();
-    load_all_types = load_all_types;
-    linux_kernel_mode = linux_kernel_mode;
+    options().load_all_types = load_all_types;
+    options().load_in_linux_kernel_mode = linux_kernel_mode;
     elf_based_reader::initialize(elf_path, debug_info_root_paths);
   }
 
@@ -385,7 +385,7 @@ public:
     // for vmlinux.ctfa should be provided with --debug-info-dir
     // option.
     for (const auto& path : debug_info_root_paths())
-      if (tools_utils::find_file_under_dir(*path, "vmlinux.ctfa", ctfa_file))
+      if (tools_utils::find_file_under_dir(path, "vmlinux.ctfa", ctfa_file))
         return true;
 
     return false;
@@ -493,7 +493,7 @@ public:
 	ctf_sym_type = lookup_symbol_in_ctf_archive(ctfa, &ctf_dict,
 						    sym_name.c_str());
 	if (ctf_sym_type == CTF_ERR)
-          continue;
+	  continue;
 
 	if (ctf_type_kind(ctf_dict, ctf_sym_type) != CTF_K_FUNCTION)
 	  {
@@ -1702,16 +1702,13 @@ lookup_symbol_in_ctf_archive(ctf_archive_t *ctfa, ctf_dict_t **ctf_dict,
 				    /*skip_parent=*/true,
 				    &ctf_err)) != nullptr)
         {
-          if ((ctf_type = ctf_lookup_by_symbol_name (fp, sym_name)) == CTF_ERR)
-            ctf_type = ctf_lookup_variable(fp, sym_name);
-
-          if (ctf_type != CTF_ERR)
-            {
-              *ctf_dict = fp;
-              break;
-            }
-          ctf_dict_close(fp);
+	  if ((ctf_type = ctf_lookup_by_symbol_name (fp, sym_name)) == CTF_ERR)
+	    ctf_type = ctf_lookup_variable(fp, sym_name);
+	  ctf_dict_close(fp);
+	  if (ctf_type != CTF_ERR)
+	    break;
         }
+      ctf_next_destroy(i);
     }
 
   return ctf_type;
@@ -1753,7 +1750,7 @@ fill_ctf_section(const Elf_Scn *elf_section, ctf_sect_t *ctf_section)
 /// @param env a libabigail IR environment.
 elf_based_reader_sptr
 create_reader(const std::string& elf_path,
-	      const vector<char**>& debug_info_root_paths,
+	      const vector<string>& debug_info_root_paths,
 	      environment& env)
 {
   reader_sptr result(new reader(elf_path,
@@ -1781,7 +1778,7 @@ create_reader(const std::string& elf_path,
 void
 reset_reader(elf_based_reader&		rdr,
 	     const std::string&	elf_path,
-	     const vector<char**>&	debug_info_root_path)
+	     const vector<string>&	debug_info_root_path)
 {
   ctf::reader& r = dynamic_cast<reader&>(rdr);
   r.initialize(elf_path, debug_info_root_path);
