@@ -7,6 +7,8 @@
 
 #include <string>
 #include <iostream>
+#include <fstream>
+#include "abg-tools-utils.h"
 
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
@@ -43,6 +45,137 @@ build_sptr<xmlChar>(xmlChar *p)
 namespace xml
 {
 using std::istream;
+using std::ifstream;
+using tools_utils::xz_decompressor_type;
+using tools_utils::file_type;
+using tools_utils::guess_file_type;
+
+// <xmlIO callbacks for xz reading support>
+
+
+/// This is an xmlIO callback function used in the libxml2 I/O input
+/// API to detect if the current handler can provider input
+/// functionality for a file designed by a path.
+///
+/// This function should return 1 iff the file contains XZ-compressed
+/// data.
+///
+/// @param filepath the path to file to consider.
+///
+/// @return 1 iff the file designated by @p filepath is XZ-compressed.
+static int
+xz_io_match_cb(const char*filepath)
+{
+  bool does_match = false;
+  file_type t = guess_file_type(filepath, /*look_through_compression=*/false);
+  if (t == tools_utils::FILE_TYPE_XZ)
+    does_match = true;
+
+  return does_match;
+}
+
+/// This is the context used by the xmlIO handler that provides input
+/// functionality to the libxml2 I/O input API for XZ-compressed XML
+/// files.
+struct xz_ctxt_type
+{
+  // The input XZ-compressed file stream.
+  std::unique_ptr<std::ifstream> input_fstream;
+  // The custom XZ-decompressor streambuf provided by tools-utils.
+  std::unique_ptr<xz_decompressor_type> decompressor_streambuf;
+  // The decompressed input stream that we can read from.
+  std::unique_ptr<std::istream> decompressed_input_stream;
+
+  xz_ctxt_type() = delete;
+
+  /// Constructor.
+  ///
+  /// @param is the XZ-compressed input file stream to consider.
+  xz_ctxt_type(std::ifstream* is)
+    : input_fstream(is),
+      decompressor_streambuf(new xz_decompressor_type(*is)),
+      decompressed_input_stream(new istream(decompressor_streambuf.get()))
+  {}
+}; // end struct  xz_ctxt_type.
+
+/// Callback used in the I/O input API of libxml2 to open a file
+/// designated by a path and containing XZ-compressed content.
+///
+/// @param filepath the path to the file to open.  The file should
+/// contain XZ-compressed data, as detected by @ref xz_io_match_cb.
+///
+/// @return a pointer to an instance of @ref xz_ctxt_type if the
+/// function could successfully open the file denoted by @p filepath.
+/// Please note that this instance of @ref xz_ctxt_type has to be
+/// deleted by @ref xz_io_close_cb.
+static void*
+xz_io_open_cb(const char* filepath)
+{
+  std::ifstream* s = new std::ifstream(filepath, ifstream::binary);
+  if (s->bad())
+    {
+      delete s;
+      return nullptr;
+    }
+
+  xz_ctxt_type *ctxt = new xz_ctxt_type(s);
+  return ctxt;
+}
+
+/// Callback used in the I/O input API of libxml2 to read and
+/// decompress data from an XZ-compressed file previously opened by
+/// @ref xz_io_open_cb.
+///
+/// @param context a pointer to the instance of @ref xz_ctxt_type
+/// returned by @ref xz_io_open_cb.  That context is used to read and
+/// decompress the XZ-compressed data coming from input file.
+///
+/// @param buffer the buffer where to copy the XZ-decompressed data.
+///
+/// @param len the length of @p buffer.
+///
+/// @return the actual number of bytes decompressed and copied into @p
+/// buffer.
+static int
+xz_io_read_cb(void* context, char *buffer, int len)
+{
+  xz_ctxt_type *ctxt = static_cast<xz_ctxt_type *>(context);
+  ctxt->decompressed_input_stream->read(buffer, len);
+  int nb_bytes_read = ctxt->decompressed_input_stream->gcount();
+  return nb_bytes_read;
+}
+
+/// Callback used in the I/O input API of libxml2 to delete the
+/// instance of @ref xz_ctxt_type created by @ref xz_io_open_cb and
+/// free its associated resources.
+///
+/// @param context the pointer to the instance of @ref xz_ctxt_type to
+/// delete.
+///
+/// @return 0 iff the operation was successful.
+static int
+xz_io_close_cb(void* context)
+{
+  xz_ctxt_type *ctxt = static_cast<xz_ctxt_type*>(context);
+  ctxt->decompressed_input_stream.reset();
+  ctxt->input_fstream->close();
+  ctxt->input_fstream.reset();
+  delete ctxt;
+  return 0;
+}
+
+// </xmlIO callbacks for xz reading support>
+
+/// The initialization function of libxml2 abstraction layer.  This
+/// function must be called prior to using any of the libxml2 capabilities.
+void
+initialize()
+{
+  LIBXML_TEST_VERSION;
+  xmlInitParser();
+  xmlRegisterInputCallbacks(xz_io_match_cb, xz_io_open_cb,
+			    xz_io_read_cb, xz_io_close_cb);
+}
 
 /// Instantiate an xmlTextReader that parses the content of an on-disk
 /// file, wrap it into a smart pointer and return it.
@@ -134,7 +267,7 @@ new_reader_from_istream(std::istream* in)
 /// @return true if the shared pointer to xmlChar contained a non NULL
 /// string, false otherwise.
 bool
-xml_char_sptr_to_string(xml_char_sptr ssptr, std::string& s)
+xml_char_sptr_to_string(xml_char_sptr& ssptr, std::string& s)
 {
   bool non_nil = false;
   if (CHAR_STR(ssptr))
