@@ -1,4 +1,4 @@
-// g++ -g -Wall -c test5-3-v1.cch
+// g++ -g -Wall -c test5-2-v1.cc
 
 struct C0
 {
@@ -9,7 +9,7 @@ struct C0
   {}
 
   virtual int vfn1() {return 0;}
-  virtual unsigned vfn2(char) {return 0;}
+  virtual int vfn2(char) {return 0;}
 };
 
 typedef C0 c0_type;
