@@ -5,15 +5,15 @@
 ///
 #include "test2.h"
 
-second_type::second_type()
+first_type::first_type()
   : member0(0),
     member1(0)
 {}
-
 namespace a
 {
-second_type*
-build_second_type()
-{return new second_type;}
+
+first_type*
+build_first_type()
+{return new first_type;}
 
 }
