@@ -122,7 +122,7 @@ InOutSpec in_out_specs[] =
     "",
     "",
     "",
-    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    abigail::tools_utils::ABIDIFF_OK,
     "data/test-abidiff-exit/test-loc-with-locs-report.txt",
     "output/test-abidiff-exit/test-loc-with-locs-report.txt"
   },
@@ -137,7 +137,7 @@ InOutSpec in_out_specs[] =
     "",
     "",
     "--no-show-locs",
-    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    abigail::tools_utils::ABIDIFF_OK,
     "data/test-abidiff-exit/test-loc-without-locs-report.txt",
     "output/test-abidiff-exit/test-loc-without-locs-report.txt"
   },
@@ -214,7 +214,8 @@ InOutSpec in_out_specs[] =
     "",
     "",
     "--leaf-changes-only",
-    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE
+    | abigail::tools_utils::ABIDIFF_ABI_INCOMPATIBLE_CHANGE,
     "data/test-abidiff-exit/test-leaf-redundant-report.txt",
     "output/test-abidiff-exit/test-leaf-redundant-report.txt"
   },
@@ -996,6 +997,21 @@ InOutSpec in_out_specs[] =
     "data/test-abidiff-exit/PR30329/PR30329-report-1.txt",
     "output/test-abidiff-exit/PR30329/PR30329-report-1.txt"
   },
+  {
+    "data/test-abidiff-exit/PR30329/old-image/usr/lib/x86_64-linux-gnu/libsqlite3.so.0.8.6",
+    "data/test-abidiff-exit/PR30329/new-image/usr/lib/x86_64-linux-gnu/libsqlite3.so.0.8.6",
+    "",
+    "",
+    "",
+    "data/test-abidiff-exit/PR30329/old-image/usr/lib/debug,data/test-abidiff-exit/PR30329/old-image/usr/lib/debug/dwz/components/sqlite.bst",
+    "data/test-abidiff-exit/PR30329/new-image/usr/lib/debug,data/test-abidiff-exit/PR30329/old-image/usr/lib/debug/dwz/components/sqlite.bst",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/PR30329/PR30329-report-1.txt",
+    "output/test-abidiff-exit/PR30329/PR30329-report-1.txt"
+  },
   {
     "data/test-abidiff-exit/PR30503/libsdl/1.2.60/lib64/libSDL-1.2.so.1.2.60",
     "data/test-abidiff-exit/PR30503/libsdl/1.2.64/lib64/libSDL-1.2.so.1.2.64",
@@ -1578,6 +1594,81 @@ InOutSpec in_out_specs[] =
     "data/test-abidiff-exit/PR28505-test-report-2.txt",
     "output/test-abidiff-exit/PR28505-test-report-2.txt"
   },
+  {
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-1.v0.o",
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-1.v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-1-report-1.txt",
+    "output/test-abidiff-exit/test-PR32902/test-PR32902-1-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-2.v0.o",
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-2.v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/test-PR32902/test-PR32902-2-report-1.txt",
+    "output/test-abidiff-exit/test-PR32902/test-PR32902-2-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/replace-dm-with-compatible-anon-dm-1-v0.o",
+    "data/test-abidiff-exit/replace-dm-with-compatible-anon-dm-1-v1.o",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_OK,
+    "data/test-abidiff-exit/replace-dm-with-compatible-anon-dm-1-report-1.txt",
+    "output/test-abidiff-exit/replace-dm-with-compatible-anon-dm-1-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR33055/old-lib.so",
+    "data/test-abidiff-exit/PR33055/new-lib.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/PR33055/PR33055-report-1.txt",
+    "output/test-abidiff-exit/PR33055/PR33055-report-1.txt"
+  },
+  {
+    "data/test-abidiff-exit/PR33055/old-lib.so",
+    "data/test-abidiff-exit/PR33055/new-lib.so",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "",
+    "--no-default-suppression --harmless",
+    abigail::tools_utils::ABIDIFF_ABI_CHANGE,
+    "data/test-abidiff-exit/PR33055/PR33055-report-2.txt",
+    "output/test-abidiff-exit/PR33055/PR33055-report-2.txt"
+  },
 #ifdef WITH_BTF
   {
     "data/test-abidiff-exit/btf/test0-v0.o",
@@ -1646,7 +1737,8 @@ main()
     in_suppression_path, abidiff_options, abidiff, cmd, diff_cmd,
     ref_diff_report_path, out_diff_report_path, in_elfv0_debug_dir,
     in_elfv1_debug_dir, in_elfv0_added_bins_dir, in_elfv1_added_bins_dir;
-  vector<string> in_elfv0_headers_dirs, in_elfv1_headers_dirs;
+  vector<string> in_elfv0_headers_dirs, in_elfv1_headers_dirs,
+    in_elfv0_debug_dirs, in_elfv1_debug_dirs;
   string source_dir_prefix = string(get_src_dir()) + "/tests/";
   string build_dir_prefix = string(get_build_dir()) + "/tests/";
 
@@ -1658,9 +1750,19 @@ main()
 	in_elfv0_debug_dir.clear();
 	in_elfv1_debug_dir.clear();
 	if (s->in_elfv0_debug_dir && strcmp(s->in_elfv0_debug_dir, ""))
-	  in_elfv0_debug_dir = source_dir_prefix + s->in_elfv0_debug_dir;
+	  {
+	    if (!split_string(s->in_elfv0_debug_dir, ",", in_elfv0_debug_dirs))
+	      in_elfv0_debug_dirs.push_back(s->in_elfv0_debug_dir);
+	    for (auto& dd : in_elfv0_debug_dirs)
+	      dd = source_dir_prefix + dd;
+	  }
 	if (s->in_elfv1_debug_dir && strcmp(s->in_elfv1_debug_dir, ""))
-	  in_elfv1_debug_dir = source_dir_prefix + s->in_elfv1_debug_dir;
+	  {
+	    if (!split_string(s->in_elfv1_debug_dir, ",", in_elfv1_debug_dirs))
+	      in_elfv1_debug_dirs.push_back(s->in_elfv1_debug_dir);
+	    for (auto& dd : in_elfv1_debug_dirs)
+	      dd = source_dir_prefix + dd;
+	  }
 	in_elfv0_headers_dirs.clear();
 	in_elfv1_headers_dirs.clear();
 	in_elfv0_added_bins_dir.clear();
@@ -1716,11 +1818,11 @@ main()
 	if (!in_elfv1_added_bins_dir.empty())
 	  abidiff += " --added-binaries-dir2 " + in_elfv1_added_bins_dir;
 
-	if (!in_elfv0_debug_dir.empty())
-	  abidiff += " --debug-info-dir1 " + in_elfv0_debug_dir;
+	for (auto& dd : in_elfv0_debug_dirs)
+	  abidiff += " --debug-info-dir1 " + dd;
 
-	if (!in_elfv1_debug_dir.empty())
-	  abidiff += " --debug-info-dir2 " + in_elfv1_debug_dir;
+	for (auto& dd : in_elfv1_debug_dirs)
+	  abidiff += " --debug-info-dir2 " + dd;
 
 	if (!in_elfv0_headers_dirs.empty())
 	  for (vector<string>::const_iterator s = in_elfv0_headers_dirs.begin();
