@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -93,6 +93,20 @@ InOutSpec in_out_specs[] =
     "data/test-diff-filter/test5-report.txt",
     "output/test-diff-filter/test5-report.txt",
   },
+  {
+    "data/test-diff-filter/test5-2-v0.o",
+    "data/test-diff-filter/test5-2-v1.o",
+    "--no-default-suppression --no-linkage-name --no-show-locs --no-redundant",
+    "data/test-diff-filter/test5-2-report.txt",
+    "output/test-diff-filter/test5-2-report.txt",
+  },
+  {
+    "data/test-diff-filter/test5-3-v0.o",
+    "data/test-diff-filter/test5-3-v1.o",
+    "--no-default-suppression --no-linkage-name --no-show-locs --no-redundant",
+    "data/test-diff-filter/test5-3-report.txt",
+    "output/test-diff-filter/test5-3-report.txt",
+  },
   {
     "data/test-diff-filter/test6-v0.o",
     "data/test-diff-filter/test6-v1.o",
@@ -836,6 +850,13 @@ InOutSpec in_out_specs[] =
     "data/test-diff-filter/test-PR29811-0-report-1.txt",
     "output/test-diff-filter/test-PR29811-0-report-1.txt",
   },
+  {
+    "data/test-diff-filter/test-multi-array1-v0.o",
+    "data/test-diff-filter/test-multi-array1-v1.o",
+    "--harmless --no-default-suppression",
+    "data/test-diff-filter/test-multi-array1.txt",
+    "output/test-diff-filter/test-multi-array1.txt",
+  },
 #ifdef WITH_CTF
   {
    "data/test-diff-filter/test-PR29811-unknown-size-array-dwarf-ctf-DWARF.o",
