@@ -87,8 +87,7 @@ struct options
   string		wrong_option;
   string		in_file_path;
   string		out_file_path;
-  vector<char*>	di_root_paths;
-  vector<char**>	prepared_di_root_paths;
+  vector<string>	di_root_paths;
   vector<string>	headers_dirs;
   vector<string>	header_files;
   vector<string>	added_bins_dirs;
@@ -134,6 +133,7 @@ struct options
   bool			annotate;
   bool			do_log;
   bool			drop_private_types;
+  bool			force_early_suppression;
   bool			drop_undefined_syms;
   bool			assume_odr_for_cplusplus;
   bool			leverage_dwarf_factorization;
@@ -181,6 +181,7 @@ struct options
       annotate(),
       do_log(),
       drop_private_types(false),
+      force_early_suppression(false),
       drop_undefined_syms(false),
       assume_odr_for_cplusplus(true),
       leverage_dwarf_factorization(true),
@@ -189,12 +190,6 @@ struct options
 
   ~options()
   {
-    for (vector<char*>::iterator i = di_root_paths.begin();
-	 i != di_root_paths.end();
-	 ++i)
-      free(*i);
-
-    prepared_di_root_paths.clear();
   }
 };
 
@@ -204,75 +199,75 @@ display_usage(const string& prog_name, ostream& out)
   emit_prefix(prog_name, out)
     << "usage: " << prog_name << " [options] [<path-to-elf-file>]\n"
     << " where options can be: \n"
-    << "  --help|-h  display this message\n"
-    << "  --version|-v  display program version information and exit\n"
+    << "  --abidiff  compare the loaded ABI against itself\n"
     << "  --abixml-version  display the version of the ABIXML ABI format\n"
-    << "  --debug-info-dir|-d <dir-path>  look for debug info under 'dir-path'\n"
-    << "  --headers-dir|--hd <path> the path to headers of the elf file\n"
-    << "  --header-file|--hf <path> the path one header of the elf file\n"
-    << "  --out-file|-o  <file-path>  write the output to 'file-path'\n"
-    << "  --noout  do not emit anything after reading the binary\n"
-    << "  --suppressions|--suppr <path> specify a suppression file\n"
-    << "  --no-architecture  do not emit architecture info in the output\n"
-    << "  --no-corpus-path  do not take the path to the corpora into account\n"
-    << "  --no-show-locs  do not show location information\n"
-    << "  --short-locs  only print filenames rather than paths\n"
-    << "  --drop-private-types  drop private types from representation\n"
-    << "  --drop-undefined-syms  drop undefined symbols from representation\n"
-    << "  --exported-interfaces-only  analyze exported interfaces only\n"
-    << "  --allow-non-exported-interfaces  analyze interfaces that "
+    << "  --add-binaries <bin1,bin2,...>  build a corpus group with "
+    "the added inaries\n"
+    << "  --allow-non-exported-interfaces analyze interfaces that"
     "might not be exported\n"
-    << "  --no-comp-dir-path  do not show compilation path information\n"
-    << "  --no-elf-needed  do not show the DT_NEEDED information\n"
-    << "  --no-write-default-sizes  do not emit pointer size when it equals"
-    " the default address size of the translation unit\n"
-    << "  --no-parameter-names  do not show names of function parameters\n"
-    << "  --type-id-style <sequence|hash>  type id style (sequence(default): "
-       "\"type-id-\" + number; hash: hex-digits)\n"
+    << "  --annotate  annotate the ABI artifacts emitted in the output\n"
+#ifdef WITH_BTF
+    << "  --btf use BTF instead of DWARF in ELF files\n"
+#endif
     << "  --check-alternate-debug-info <elf-path>  check alternate debug info "
     "of <elf-path>\n"
     << "  --check-alternate-debug-info-base-name <elf-path>  check alternate "
     "debug info of <elf-path>, and show its base name\n"
-    << "  --load-all-types  read all types including those not reachable from "
-    "exported declarations\n"
-    << "  --no-load-undefined-interfaces  do not consider undefined "
-    "interfaces from the binary"
-    << "  --no-linux-kernel-mode  don't consider the input binary as "
-       "a Linux Kernel binary\n"
-    << "  --kmi-whitelist|--kmi-stablelist|-w  path to a linux kernel "
-    "abi whitelist\n"
-    << "  --linux-tree|--lt  emit the ABI for the union of a "
-    "vmlinux and its modules\n"
-    << "  --vmlinux <path>  the path to the vmlinux binary to consider to emit "
-       "the ABI of the union of vmlinux and its modules\n"
-    << "  --abidiff  compare the loaded ABI against itself\n"
-    << "  --add-binaries <bin1,bin2,...>  build a corpus group with "
-    "the added inaries\n"
-    << "  --follow-dependencies  build a corpus group with the dependencies\n"
-    << "  --list-dependencies  list the dependencies of a given binary\n"
-    << "  --added-binaries-dir|--abd <dir-of-deps>  where to look for dependencies "
-    "or added binaries\n"
+#ifdef WITH_CTF
+    << "  --ctf use CTF instead of DWARF in ELF files\n"
+#endif
 #ifdef WITH_DEBUG_SELF_COMPARISON
     << "  --debug-abidiff  debug the process of comparing the loaded ABI against itself\n"
 #endif
+    << "  --debug-info-dir|-d <dir-path>  look for debug info under 'dir-path'\n"
 #ifdef WITH_DEBUG_TYPE_CANONICALIZATION
     << "  --debug-tc  debug the type canonicalization process\n"
     << "  --debug-dc  debug the DIE canonicalization process\n"
 #endif
-#ifdef WITH_CTF
-    << "  --ctf use CTF instead of DWARF in ELF files\n"
-#endif
-    << "  --no-leverage-dwarf-factorization  do not use DWZ optimisations to "
-    "speed-up the analysis of the binary\n"
+    << "  --drop-undefined-syms  drop undefined symbols from representation\n"
+    << "  --exported-interfaces-only  analyze exported interfaces only\n"
+    << "  --follow-dependencies  build a corpus group with the dependencies\n"
+    << "  --follow-dependencies  build a corpus group with the dependencies\n"
+    << "  --force-early-suppression  drop IR nodes that match suppression specifications\n"
+    << "  --headers-dir|--hd <path> the path to headers of the elf file\n"
+    << "  --header-file|--hf <path> the path one header of the elf file\n"
+    << "  --help|-h  display this message\n"
+    << "  --kmi-whitelist|--kmi-stablelist|-w  path to a linux kernel "
+    "abi whitelist\n"
+    << "  --list-dependencies  list the dependencies of a given binary\n"
+    << "  --added-binaries-dir|--abd <dir-of-deps>  where to look for dependencies "
+    "or added binaries\n"
+    << "  --linux-tree|--lt  emit the ABI for the union of a "
+    "vmlinux and its modules\n"
+    << "  --load-all-types  read all types including those not reachable from "
+    "exported declarations\n"
+    << "  --no-architecture  do not emit architecture info in the output\n"
     << "  --no-assume-odr-for-cplusplus  do not assume the ODR to speed-up the "
     "analysis of the binary\n"
-#ifdef WITH_BTF
-    << "  --btf use BTF instead of DWARF in ELF files\n"
-#endif
-    << "  --annotate  annotate the ABI artifacts emitted in the output\n"
+    << "  --no-comp-dir-path  do not show compilation path information\n"
+    << "  --no-corpus-path  do not take the path to the corpora into account\n"
+    << "  --no-elf-needed  do not show the DT_NEEDED information\n"
+    << "  --no-leverage-dwarf-factorization  do not use DWZ optimisations to "
+    "speed-up the analysis of the binary\n"
+    << "  --no-linux-kernel-mode  don't consider the input binary as "
+       "a Linux Kernel binary\n"
+    << "  --no-load-undefined-interfaces  do not consider undefined "
+    "interfaces from the binary"
+    << "  --no-parameter-names  do not show names of function parameters\n"
+    << "  --no-show-locs  do not show location information\n"
+    << "  --no-write-default-sizes  do not emit pointer size when it equals"
+    " the default address size of the translation unit\n"
+    << "  --noout  do not emit anything after reading the binary\n"
+    << "  --out-file|-o  <file-path>  write the output to 'file-path'\n"
+    << "  --short-locs  only print filenames rather than paths\n"
+    << "  --suppressions|--suppr <path> specify a suppression file\n"
+    << "  --type-id-style <sequence|hash>  type id style (sequence(default): "
+       "\"type-id-\" + number; hash: hex-digits)\n"
     << "  --stats  show statistics about various internal stuff\n"
-    << "  --verbose show verbose messages about internal stuff\n";
-  ;
+    << "  --verbose show verbose messages about internal stuff\n"
+    << "  --version|-v  display program version information and exit\n"
+    << "  --vmlinux <path>  the path to the vmlinux binary to consider to emit "
+       "the ABI of the union of vmlinux and its modules\n" ;
 }
 
 static bool
@@ -305,7 +300,7 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  // elfutils wants the root path to the debug info to be
 	  // absolute.
 	  opts.di_root_paths.push_back
-	    (abigail::tools_utils::make_path_absolute_to_be_freed(argv[i + 1]));
+	    (abigail::tools_utils::make_path_absolute(string(argv[i + 1])));
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--headers-dir")
@@ -315,6 +310,10 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  if (j >= argc)
 	    return false;
 	  opts.headers_dirs.push_back(argv[j]);
+	  // The user is indirectly defining private types so she
+	  // really wants those types to be removed from the ABIXML.
+	  // So let's drop them from the IR.
+	  opts.drop_private_types = true;
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--added-binaries-dir")
@@ -333,6 +332,10 @@ parse_command_line(int argc, char* argv[], options& opts)
 	  if (j >= argc)
 	    return false;
 	  opts.header_files.push_back(argv[j]);
+	  // The user is indirectly defining private types so she
+	  // really wants those types to be removed from the ABIXML.
+	  // So let's drop them from the IR.
+	  opts.drop_private_types = true;
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--out-file")
@@ -455,6 +458,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	opts.load_undefined_interfaces = false;
       else if (!strcmp(argv[i], "--drop-private-types"))
 	opts.drop_private_types = true;
+      else if (!strcmp(argv[i], "--force-early-suppression"))
+	opts.force_early_suppression = true;
       else if (!strcmp(argv[i], "--drop-undefined-syms"))
 	opts.drop_undefined_syms = true;
       else if (!strcmp(argv[i], "--exported-interfaces-only"))
@@ -585,6 +590,13 @@ set_suppressions(abigail::elf_based_reader& rdr, options& opts)
        ++i)
     read_suppressions(*i, supprs);
 
+  if (opts.force_early_suppression)
+    // User asked to unconditionally drop suppressed artifacts from
+    // the IR.  Let's drop all nodes matched by suppression
+    // specifications from the IR.
+    for (auto& s : supprs)
+      s->set_drops_artifact_from_ir(true);
+
   suppression_sptr suppr =
     abigail::tools_utils::gen_suppr_spec_from_headers(opts.headers_dirs,
 						      opts.header_files);
@@ -789,7 +801,7 @@ load_corpus_and_write_abixml(char* argv[],
   // specfied in opts ...
   abigail::elf_based_reader_sptr reader =
     create_best_elf_based_reader(opts.in_file_path,
-				 opts.prepared_di_root_paths,
+				 opts.di_root_paths,
 				 env, requested_fe_kind,
 				 opts.load_all_types,
 				 opts.linux_kernel_mode);
@@ -865,7 +877,7 @@ load_corpus_and_write_abixml(char* argv[],
 	      emit_prefix(argv[0], cerr)
 		<< "Could not read debug info for '" << opts.in_file_path
 		<< "' from debug info root directory '";
-	      for (vector<char*>::const_iterator i =
+	      for (vector<string>::const_iterator i =
 		     opts.di_root_paths.begin();
 		   i != opts.di_root_paths.end();
 		   ++i)
@@ -929,7 +941,7 @@ load_corpus_and_write_abixml(char* argv[],
       // corpus group.
 
       // If a corpus_group already exists, use that one ...
-      if (!corp_group->is_empty())
+      if (corp_group && !corp_group->is_empty())
 	add_dependencies_into_corpus_group(reader, *corp,
 					   opts.added_bins_dirs,
 					   *corp_group);
@@ -1127,23 +1139,13 @@ load_kernel_corpus_group_and_write_abixml(char* argv[],
   return exit_code;
 }
 
-/// Convert options::di_root_paths into
-/// options::prepared_di_root_paths which is the suitable type format
-/// that the dwarf_reader expects.
-///
-/// @param o the options to consider.
-static void
-prepare_di_root_paths(options& o)
-{
-  tools_utils::convert_char_stars_to_char_star_stars(o.di_root_paths,
-						     o.prepared_di_root_paths);
-}
-
 int
 main(int argc, char* argv[])
 {
   options opts;
 
+  abigail::tools_utils::initialize();
+
   if (!parse_command_line(argc, argv, opts)
       || (opts.in_file_path.empty()
 	  && !opts.display_version
@@ -1184,8 +1186,6 @@ main(int argc, char* argv[])
 	return 1;
     }
 
-  prepare_di_root_paths(opts);
-
   if (!maybe_check_suppression_files(opts))
     return 1;
 
