@@ -41,111 +41,20 @@
 /*
  * pinstance.c
  *
- * This file implements the NSSCKMDInstance object for the
+ * This file implements the NSSCKMDInstance object for the 
  * "PEM objects" cryptoki module.
  */
 
 static PRBool pemInitialized = PR_FALSE;
 
-pemInternalObject **pem_objs;
+pemInternalObject **gobj;
 int pem_nobjs = 0;
 int token_needsLogin[NUM_SLOTS];
-PLHashTable *nicknameHashTable = NULL;
 
-static void *strcpy_AllocTable(void *pool, PRSize size) {
-    return PORT_Alloc(size);
-}
-static void strcpy_FreeTable(void *pool, void *item) {
-    PORT_Free(item);
-}
-static PLHashEntry* strcpy_AllocEntry(void *pool, const void *key) {
-    return PORT_New(PLHashEntry);
-}
-static void strcpy_FreeEntry(void *pool, PLHashEntry *he, PRUintn flag) {
-    PORT_Free(he->value);
-    if (flag == HT_FREE_ENTRY) {
-        PORT_Free(he);
-    }
-}
-static PLHashAllocOps strcpy_AllocOps = {
-    strcpy_AllocTable, strcpy_FreeTable, strcpy_AllocEntry, strcpy_FreeEntry
-};
+PRInt32 size = 0;
+PRInt32 count = 0;
 
-void releasePEMNickname(char *nickname)
-{
-    if (nickname) {
-        PL_HashTableRemove(nicknameHashTable, nickname);
-    }
-}
-
-/* Function takes a filename, will use the base name (without directory),
- * and append numbers until a unique nickname is found (or until we give up).
- * The caller owns the returned nickname, which should be destroyed
- * using nss_ZFreeIf().
- * In addition, once the object associated to the nickname is freed,
- * the nickname should be released using releasePEMNickname(),
- * in order to remove the entry from the hashtable of unique nicknames.
- * Caller may provide an optional nonzero numeric parameter that will be used
- * as a starting point when creating the suffix. */
-char *getUniquePEMNicknameFromFilename(const char *filename, int start_suffix)
-{
-    const char *basename;
-    size_t len_basename;
-
-    char *returned_name = NULL;
-    const int max_conflict_resolution_attempts = 999999999;
-    const size_t max_digits_for_conflict_resolution = 9;
-    int conflict_suffix = 0;
-
-    if (!filename)
-        return NULL;
-
-    if (start_suffix < 0 || start_suffix > max_conflict_resolution_attempts)
-        start_suffix = 0;
-
-    conflict_suffix = start_suffix;
-
-    basename = strrchr(filename, PR_GetDirectorySeparator());
-    if (basename)
-        basename++;
-    else
-        basename = filename;
-
-    len_basename = strlen(basename);
-    if (!len_basename) {
-        return NULL;
-    }
-
-    returned_name = (char *) nss_ZAlloc(NULL,
-                        len_basename + max_digits_for_conflict_resolution + 1);
-    if (!returned_name)
-        return NULL;
-
-    strcpy(returned_name, basename);
-    /* snprintf won't write a zero terminator if it's using the max length */
-    returned_name[len_basename + max_digits_for_conflict_resolution] = '\0';
-    do {
-        /* Skip suffix "0". */
-        if (conflict_suffix > 0) {
-            snprintf(returned_name + len_basename,
-                     max_digits_for_conflict_resolution,
-                     "%d", conflict_suffix);
-        }
-
-        if (PL_HashTableLookup(nicknameHashTable, returned_name) == NULL) {
-            /* no entry found for this name, good to use */
-            /* Our hashtable's strcpy_AllocOps will PORT_Free the value */
-            char *copy_for_hashtable = PORT_Strdup(returned_name);
-            PL_HashTableAdd(nicknameHashTable, copy_for_hashtable, copy_for_hashtable);
-
-            return returned_name;
-        }
-
-    } while (++conflict_suffix <= max_conflict_resolution_attempts);
-
-    nss_ZFreeIf(returned_name);
-    return NULL;
-}
+#define PEM_ITEM_CHUNK  512
 
 /*
  * simple cert decoder to avoid the cost of asn1 engine
@@ -293,7 +202,7 @@ assignObjectID(pemInternalObject *o, int objid)
 static pemInternalObject *
 CreateObject(CK_OBJECT_CLASS objClass,
              pemObjectType type, SECItem * certDER,
-             SECItem * keyDER, const char *nickname,
+             SECItem * keyDER, char *filename,
              int objid, CK_SLOT_ID slotID)
 {
     pemInternalObject *o;
@@ -303,12 +212,19 @@ CreateObject(CK_OBJECT_CLASS objClass,
     SECItem derSN;
     SECItem valid;
     SECItem subjkey;
+    char *nickname;
 
     o = nss_ZNEW(NULL, pemInternalObject);
     if ((pemInternalObject *) NULL == o) {
         return NULL;
     }
 
+    nickname = strrchr(filename, '/');
+    if (nickname)
+        nickname++;
+    else
+        nickname = filename;
+
     switch (objClass) {
     case CKO_CERTIFICATE:
         plog("Creating cert nick %s id %d in slot %ld\n", nickname, objid, slotID);
@@ -317,6 +233,7 @@ CreateObject(CK_OBJECT_CLASS objClass,
     case CKO_PRIVATE_KEY:
         plog("Creating key id %d in slot %ld\n", objid, slotID);
         memset(&o->u.key, 0, sizeof(o->u.key));
+        nickname = filename;
         break;
     case CKO_NETSCAPE_TRUST:
         plog("Creating trust nick %s id %d in slot %ld\n", nickname, objid, slotID);
@@ -449,7 +366,7 @@ LinkSharedKeyObject(int oldKeyIdx, int newKeyIdx)
     int i;
     for (i = 0; i < pem_nobjs; i++) {
         CK_RV rv;
-        pemInternalObject *obj = pem_objs[i];
+        pemInternalObject *obj = gobj[i];
         if (NULL == obj)
             continue;
 
@@ -468,26 +385,33 @@ LinkSharedKeyObject(int oldKeyIdx, int newKeyIdx)
 pemInternalObject *
 AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
                   pemObjectType type, SECItem * certDER,
-                  SECItem * keyDER, const char *nickname,
+                  SECItem * keyDER, char *filename,
                   int objid, CK_SLOT_ID slotID, PRBool *pAdded)
 {
     int i;
 
+    /* FIXME: copy-pasted from CreateObject */
+    const char *nickname = strrchr(filename, '/');
+    if (nickname && CKO_PRIVATE_KEY != objClass)
+        nickname++;
+    else
+        nickname = filename;
+
     if (pAdded)
         *pAdded = PR_FALSE;
 
-    /* first look for the object in pem_objs, it might be already there */
+    /* first look for the object in gobj, it might be already there */
     for (i = 0; i < pem_nobjs; i++) {
-        if (NULL == pem_objs[i])
+        if (NULL == gobj[i])
             continue;
 
         /* Comparing DER encodings is dependable and frees the PEM module
          * from having to require clients to provide unique nicknames.
          */
-        if ((pem_objs[i]->objClass == objClass)
-                && (pem_objs[i]->type == type)
-                && (pem_objs[i]->slotID == slotID)
-                && derEncodingsMatch(objClass, pem_objs[i], certDER, keyDER)) {
+        if ((gobj[i]->objClass == objClass)
+                && (gobj[i]->type == type)
+                && (gobj[i]->slotID == slotID)
+                && derEncodingsMatch(objClass, gobj[i], certDER, keyDER)) {
 
             /* While adding a client certificate we (wrongly?) assumed that the
              * key object will follow right after the cert object.  However, if
@@ -497,30 +421,36 @@ AddObjectIfNeeded(CK_OBJECT_CLASS objClass,
             LinkSharedKeyObject(pem_nobjs, i);
 
             plog("AddObjectIfNeeded: re-using internal object #%i\n", i);
-            pem_objs[i]->refCount ++;
-            return pem_objs[i];
+            gobj[i]->refCount ++;
+            return gobj[i];
         }
     }
 
     /* object not found, we need to create it */
     pemInternalObject *io = CreateObject(objClass, type, certDER, keyDER,
-                                         nickname, objid, slotID);
+                                         filename, objid, slotID);
     if (io == NULL)
         return NULL;
 
     /* initialize pointers to functions */
     pem_CreateMDObject(NULL, io, NULL);
 
-    io->gobjIndex = pem_nobjs;
+    io->gobjIndex = count;
 
     /* add object to global array */
-    if (pem_objs)
-        pem_objs = nss_ZREALLOCARRAY(pem_objs, pemInternalObject *, pem_nobjs+1);
-    else
-        pem_objs = nss_ZNEWARRAY(NULL, pemInternalObject *, 1);
-    if (!pem_objs)
-        return NULL;
-    pem_objs[pem_nobjs] = io;
+    if (count >= size) {
+        gobj = gobj ?
+            nss_ZREALLOCARRAY(gobj, pemInternalObject *,
+                    (size+PEM_ITEM_CHUNK) ) :
+            nss_ZNEWARRAY(NULL, pemInternalObject *,
+                    (size+PEM_ITEM_CHUNK) ) ;
+
+        if ((pemInternalObject **)NULL == gobj)
+            return NULL;
+        size += PEM_ITEM_CHUNK;
+    }
+    gobj[count] = io;
+    count++;
     pem_nobjs++;
 
     if (pAdded)
@@ -534,14 +464,13 @@ CK_RV
 AddCertificate(char *certfile, char *keyfile, PRBool cacert,
                CK_SLOT_ID slotID)
 {
-    pemInternalObject *o = NULL;
+    pemInternalObject *o;
     CK_RV error = 0;
-    int objid, i = 0;
+    int objid, i;
     int nobjs = 0;
     SECItem **objs = NULL;
     char *ivstring = NULL;
     int cipher;
-    char *nickname = NULL;
 
     /* TODO: Fix discrepancy between our usage of the return value as
      * as an int (a count) and the declaration as a SECStatus. */
@@ -554,43 +483,38 @@ AddCertificate(char *certfile, char *keyfile, PRBool cacert,
     /* For now load as many certs as are in the file for CAs only */
     if (cacert) {
         for (i = 0; i < nobjs; i++) {
+            char nickname[1024];
             objid = pem_nobjs + 1;
 
-            nickname = getUniquePEMNicknameFromFilename(certfile, i);
-            if (!nickname) {
+            snprintf(nickname, 1024, "%s - %d", certfile, i);
+
+            o = AddObjectIfNeeded(CKO_CERTIFICATE, pemCert, objs[i], NULL,
+                                   nickname, 0, slotID, NULL);
+            if (o == NULL) {
                 error = CKR_GENERAL_ERROR;
                 goto loser;
             }
 
-            o = AddObjectIfNeeded(CKO_CERTIFICATE, pemCert, objs[i], NULL,
+            /* Add the CA trust object */
+            o = AddObjectIfNeeded(CKO_NETSCAPE_TRUST, pemTrust, objs[i], NULL,
                                    nickname, 0, slotID, NULL);
-            if (o != NULL) {
-                /* Add the CA trust object */
-                o = AddObjectIfNeeded(CKO_NETSCAPE_TRUST, pemTrust, objs[i], NULL,
-                                       nickname, 0, slotID, NULL);
-            }
-            nss_ZFreeIf(nickname);
-            nickname = NULL;
             if (o == NULL) {
                 error = CKR_GENERAL_ERROR;
                 goto loser;
             }
         }                       /* for */
     } else {
-        PRBool found_error = PR_FALSE;
-
         objid = pem_nobjs + 1;
-
-        nickname = getUniquePEMNicknameFromFilename(certfile, i);
-        if (!nickname) {
+        o = AddObjectIfNeeded(CKO_CERTIFICATE, pemCert, objs[0], NULL, certfile,
+                              objid, slotID, NULL);
+        if (o == NULL) {
             error = CKR_GENERAL_ERROR;
             goto loser;
         }
 
-        o = AddObjectIfNeeded(CKO_CERTIFICATE, pemCert, objs[0], NULL, nickname,
-                              objid, slotID, NULL);
+        o = NULL;
 
-        if (o != NULL && keyfile != NULL) { /* add the private key */
+        if (keyfile) {          /* add the private key */
             SECItem **keyobjs = NULL;
             int kobjs = 0;
             /* TODO: Fix discrepancy between our usage of the return value as
@@ -599,127 +523,27 @@ AddCertificate(char *certfile, char *keyfile, PRBool cacert,
                 (int) ReadDERFromFile(&keyobjs, keyfile, PR_TRUE, &cipher,
                                 &ivstring, PR_FALSE);
             if (kobjs < 1) {
-                found_error = PR_TRUE;
-            } else {
-                o = AddObjectIfNeeded(CKO_PRIVATE_KEY, pemBareKey, objs[0],
-                                      keyobjs[0], nickname, objid, slotID, NULL);
+                error = CKR_GENERAL_ERROR;
+                goto loser;
+            }
+            o = AddObjectIfNeeded(CKO_PRIVATE_KEY, pemBareKey, objs[0],
+                                  keyobjs[0], certfile, objid, slotID, NULL);
+            if (o == NULL) {
+                error = CKR_GENERAL_ERROR;
+                goto loser;
             }
-        }
-
-        nss_ZFreeIf(nickname);
-        nickname = NULL;
-
-        if (found_error || o == NULL) {
-            error = CKR_GENERAL_ERROR;
-            goto loser;
         }
     }
 
-    nss_ZFreeIf(nickname);
     nss_ZFreeIf(objs);
     return CKR_OK;
 
   loser:
     nss_ZFreeIf(objs);
     nss_ZFreeIf(o);
-    nss_ZFreeIf(nickname);
     return error;
 }
 
-#define DynPtrList_default_capacity 4
-#define DynPtrList_default_realloc_factor 2
-
-static void*
-myDynPtrListAllocWrapper(size_t bytes)
-{
-    return nss_ZAlloc(NULL, bytes);
-}
-
-static void*
-myDynPtrListReallocWrapper(void *ptr, size_t bytes)
-{
-    return nss_ZRealloc(ptr, bytes);
-}
-
-static void
-myDynPtrListFreeWrapper(void *ptr)
-{
-    nss_ZFreeIf(ptr);
-}
-
-/* returns NULL on failure. returns dpl if init was successful. */
-void*
-pem_InitDynPtrList(DynPtrList *dpl, DynPtrListAllocFunction a,
-                  DynPtrListReallocFunction r, DynPtrListFreeFunction f)
-{
-    if (!dpl)
-        return NULL;
-
-    dpl->entries = 0;
-    dpl->capacity = DynPtrList_default_capacity;
-
-    dpl->alloc_function = a;
-    dpl->realloc_function = r;
-    dpl->free_function = f;
-
-    dpl->pointers = (*dpl->alloc_function)(dpl->capacity * sizeof(void*));
-    if (!dpl->pointers)
-        return NULL;
-
-    return dpl;
-}
-
-void
-pem_FreeDynPtrList(DynPtrList *dpl)
-{
-    size_t i;
-    for (i = 0; i < dpl->entries; ++i) {
-        (*dpl->free_function)(dpl->pointers[i]);
-    }
-    nss_ZFreeIf(dpl->pointers);
-    dpl->pointers = NULL;
-    dpl->capacity = 0;
-    dpl->entries = 0;
-}
-
-/* returns NULL on failure. Returns str if it could be added.*/
-void*
-pem_AddToDynPtrList(DynPtrList *dpl, char *ptr)
-{
-    const size_t max_size_t = ((size_t) -1);
-
-    if (!dpl->capacity)
-        return NULL; /* dpl not initialized */
-
-    if (dpl->capacity == dpl->entries) {
-        /* capacity reached, must grow */
-        void **new_pointers = NULL;
-        size_t new_capacity;
-
-        if ( (((double)max_size_t) / dpl->capacity) < DynPtrList_default_realloc_factor) {
-            new_capacity = max_size_t;
-        } else {
-            new_capacity = dpl->capacity * DynPtrList_default_realloc_factor;
-        }
-
-        if (dpl->capacity == new_capacity) {
-            return NULL; /* cannot grow */
-        }
-
-        new_pointers = (*dpl->realloc_function)(dpl->pointers, new_capacity);
-        if (new_pointers == dpl->pointers) {
-            return NULL; /* cannot grow */
-        }
-
-        dpl->pointers = new_pointers;
-        dpl->capacity = new_capacity;
-    }
-
-    dpl->pointers[dpl->entries] = ptr;
-    ++dpl->entries;
-    return ptr;
-}
-
 CK_RV
 pem_Initialize
 (
@@ -730,8 +554,9 @@ pem_Initialize
 {
     CK_RV rv;
     /* parse the initialization string */
+    char **certstrings = NULL;
     char *modparms = NULL;
-    DynPtrList certstrings;
+    PRInt32 numcerts = 0;
     PRBool status, error = PR_FALSE;
     int i;
     CK_C_INITIALIZE_ARGS_PTR modArgs = NULL;
@@ -750,20 +575,13 @@ pem_Initialize
 
     RNG_RNGInit();
 
-    open_nss_pem_log();
+    open_log();
 
     plog("pem_Initialize\n");
 
-    nicknameHashTable = PL_NewHashTable(0, PL_HashString, PL_CompareStrings,
-                                        PL_CompareStrings, &strcpy_AllocOps, NULL);
-    if (!nicknameHashTable) {
-        return CKR_HOST_MEMORY;
-    }
-
     if (!modArgs || !modArgs->LibraryParameters) {
         goto done;
     }
-
     modparms = (char *) modArgs->LibraryParameters;
     plog("Initialized with %s\n", modparms);
 
@@ -779,40 +597,38 @@ pem_Initialize
      *  /etc/certs/server.pem;/etc/certs/server.key /etc/certs/ca.pem
      *
      */
-    pem_InitDynPtrList(&certstrings, myDynPtrListAllocWrapper,
-                      myDynPtrListReallocWrapper, myDynPtrListFreeWrapper);
-    status = pem_ParseString(modparms, ' ', &certstrings);
+    status =
+        pem_ParseString(modparms, ' ', &numcerts,
+                        &certstrings);
     if (status == PR_FALSE) {
         return CKR_ARGUMENTS_BAD;
     }
 
-    for (i = 0; i < certstrings.entries && error != PR_TRUE; i++) {
-        char *cert = (char*)certstrings.pointers[i];
-        DynPtrList certattrs;
-
-        pem_InitDynPtrList(&certattrs, myDynPtrListAllocWrapper,
-                          myDynPtrListReallocWrapper, myDynPtrListFreeWrapper);
-        status = pem_ParseString(cert, ';', &certattrs);
+    for (i = 0; i < numcerts && error != PR_TRUE; i++) {
+        char *cert = certstrings[i];
+        PRInt32 attrcount = 0;
+        char **certattrs = NULL;
+        status = pem_ParseString(cert, ';', &attrcount, &certattrs);
         if (status == PR_FALSE) {
             error = PR_TRUE;
             break;
         }
 
         if (error == PR_FALSE) {
-            if (certattrs.entries == 1) /* CA certificate */
-                rv = AddCertificate(certattrs.pointers[0], NULL, PR_TRUE, i);
+            if (attrcount == 1) /* CA certificate */
+                rv = AddCertificate(certattrs[0], NULL, PR_TRUE, 0);
             else
-                rv = AddCertificate(certattrs.pointers[0], certattrs.pointers[1],
-                                    PR_FALSE, i);
+                rv = AddCertificate(certattrs[0], certattrs[1], PR_FALSE,
+                                    0);
 
             if (rv != CKR_OK) {
                 error = PR_TRUE;
                 status = PR_FALSE;
             }
         }
-        pem_FreeDynPtrList(&certattrs);
+        pem_FreeParsedStrings(attrcount, certattrs);
     }
-    pem_FreeDynPtrList(&certstrings);
+    pem_FreeParsedStrings(numcerts, certstrings);
 
     if (status == PR_FALSE) {
         return CKR_ARGUMENTS_BAD;
@@ -839,13 +655,16 @@ pem_Finalize
     if (!pemInitialized)
         return;
 
-    nss_ZFreeIf(pem_objs);
-    pem_objs = NULL;
+    nss_ZFreeIf(gobj);
+    gobj = NULL;
+
     pem_nobjs = 0;
-    if (nicknameHashTable)
-        PL_HashTableDestroy(nicknameHashTable);
+    size = 0;
+    count = 0;
 
     PR_AtomicSet(&pemInitialized, PR_FALSE);
+
+    return;
 }
 
 /*
