@@ -31,7 +31,6 @@
 #include "spice.h"
 #include "red_common.h"
 #include "demarshallers.h"
-#include "reds_stream.h"
 
 #define MAX_SEND_BUFS 1000
 #define CLIENT_ACK_WINDOW 20
@@ -127,6 +126,12 @@ typedef struct OutgoingHandler {
 
 /* Red Channel interface */
 
+typedef struct BufDescriptor {
+    uint32_t size;
+    uint8_t *data;
+} BufDescriptor;
+
+typedef struct RedsStream RedsStream;
 typedef struct RedChannel RedChannel;
 typedef struct RedChannelClient RedChannelClient;
 typedef struct RedClient RedClient;
@@ -218,7 +223,7 @@ typedef struct RedChannelCapabilities {
     uint32_t *caps;
 } RedChannelCapabilities;
 
-int test_capability(const uint32_t *caps, int num_caps, uint32_t cap);
+int test_capabilty(const uint32_t *caps, int num_caps, uint32_t cap);
 
 typedef struct RedChannelClientLatencyMonitor {
     int state;
@@ -303,7 +308,7 @@ struct RedChannel {
 
     RingItem link; // channels link for reds
 
-    const SpiceCoreInterface *core;
+    SpiceCoreInterface *core;
     int handle_acks;
 
     // RedChannel will hold only connected channel clients (logic - when pushing pipe item to all channel clients, there
@@ -348,7 +353,7 @@ struct RedChannel {
 /* if one of the callbacks should cause disconnect, use red_channel_shutdown and don't
  * explicitly destroy the channel */
 RedChannel *red_channel_create(int size,
-                               const SpiceCoreInterface *core,
+                               SpiceCoreInterface *core,
                                uint32_t type, uint32_t id,
                                int handle_acks,
                                channel_handle_message_proc handle_message,
@@ -358,7 +363,7 @@ RedChannel *red_channel_create(int size,
 /* alternative constructor, meant for marshaller based (inputs,main) channels,
  * will become default eventually */
 RedChannel *red_channel_create_parser(int size,
-                               const SpiceCoreInterface *core,
+                               SpiceCoreInterface *core,
                                uint32_t type, uint32_t id,
                                int handle_acks,
                                spice_parse_channel_func_t parser,
