@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef _TBB_arena_H
@@ -34,70 +26,61 @@
 
 #include "tbb/tbb_machine.h"
 
-#if !__TBB_CPU_CTL_ENV_PRESENT
-    #include <fenv.h>
-    typedef fenv_t __TBB_cpu_ctl_env_t;
-#endif /* !__TBB_CPU_CTL_ENV_PRESENT */
-
 #include "scheduler_common.h"
 #include "intrusive_list.h"
 #include "task_stream.h"
 #include "../rml/include/rml_tbb.h"
 #include "mailbox.h"
 #include "observer_proxy.h"
+#include "market.h"
+#include "governor.h"
 #if __TBB_TASK_ARENA
 #include "concurrent_monitor.h"
 #endif
 
 namespace tbb {
 
-namespace interface6 {
-class task_arena;
-}
 class task_group_context;
 class allocate_root_with_context_proxy;
 
 namespace internal {
 
-class task_scheduler_observer_v3;
-class governor;
-class arena;
-template<typename SchedulerTraits> class custom_scheduler;
-
-class market;
-
 //! arena data except the array of slots
 /** Separated in order to simplify padding. 
     Intrusive list node base class is used by market to form a list of arenas. **/
-struct arena_base : intrusive_list_node {
-    //! Market owning this arena
-    market* my_market;
-
-    //! Maximal currently busy slot.
-    atomic<unsigned> my_limit;
-
-    //! Number of slots in the arena
-    unsigned my_num_slots;
-
-    //! Number of workers requested by the master thread owning the arena
-    unsigned my_max_num_workers;
-
-    //! Number of workers that are currently requested from the resource manager
-    int my_num_workers_requested;
-
+struct arena_base : padded<intrusive_list_node> {
     //! Number of workers that have been marked out by the resource manager to service the arena
-    unsigned my_num_workers_allotted;
+    unsigned my_num_workers_allotted;   // heavy use in stealing loop
 
     //! References of the arena
     /** Counts workers and master references separately. Bit 0 indicates reference from implicit
         master or explicit task_arena; the next bits contain number of workers servicing the arena.*/
-    atomic<unsigned> my_references;
+    atomic<unsigned> my_references;     // heavy use in stealing loop
 
-    //! ABA prevention marker
-    uintptr_t my_aba_epoch;
+#if __TBB_TASK_PRIORITY
+    //! Highest priority of recently spawned or enqueued tasks.
+    volatile intptr_t my_top_priority;  // heavy use in stealing loop
+#endif /* !__TBB_TASK_PRIORITY */
 
-    //! FPU control settings of arena's master thread captured at the moment of arena instantiation.
-    __TBB_cpu_ctl_env_t my_cpu_ctl_env;
+    //! Maximal currently busy slot.
+    atomic<unsigned> my_limit;          // heavy use in stealing loop
+
+    //! Task pool for the tasks scheduled via task::enqueue() method
+    /** Such scheduling guarantees eventual execution even if
+        - new tasks are constantly coming (by extracting scheduled tasks in
+          relaxed FIFO order);
+        - the enqueuing thread does not call any of wait_for_all methods. **/
+#if __TBB_TASK_PRIORITY
+    task_stream<num_priority_levels> my_task_stream; // heavy use in stealing loop
+#else /* !__TBB_TASK_PRIORITY */
+    task_stream<1>                   my_task_stream; // heavy use in stealing loop
+#endif /* !__TBB_TASK_PRIORITY */
+
+    //! Number of workers that are currently requested from the resource manager
+    int my_num_workers_requested;
+
+    //! Number of workers requested by the master thread owning the arena
+    unsigned my_max_num_workers;
 
 #if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
     int my_num_workers_present;
@@ -110,17 +93,12 @@ struct arena_base : intrusive_list_node {
         my_pool_state to be unsigned. */
     tbb::atomic<uintptr_t> my_pool_state;
 
-#if __TBB_TASK_GROUP_CONTEXT
-    //! Default task group context.
-    /** Used by root tasks allocated directly by the master thread (not from inside
-        a TBB task) without explicit context specification. **/
-    task_group_context* my_default_ctx;
-#endif /* __TBB_TASK_GROUP_CONTEXT */
+#if __TBB_SCHEDULER_OBSERVER
+    //! List of local observers attached to this arena.
+    observer_list my_observers;
+#endif /* __TBB_SCHEDULER_OBSERVER */
 
 #if __TBB_TASK_PRIORITY
-    //! Highest priority of recently spawned or enqueued tasks.
-    volatile intptr_t my_top_priority;
-
     //! Lowest normalized priority of available spawned or enqueued tasks.
     intptr_t my_bottom_priority;
 
@@ -135,32 +113,35 @@ struct arena_base : intrusive_list_node {
     //! Counter used to track the occurrence of recent orphaning and re-sharing operations.
     tbb::atomic<uintptr_t> my_abandonment_epoch;
 
-    //! Task pool for the tasks scheduled via task::enqueue() method
-    /** Such scheduling guarantees eventual execution even if
-        - new tasks are constantly coming (by extracting scheduled tasks in 
-          relaxed FIFO order);
-        - the enqueuing thread does not call any of wait_for_all methods. **/
-    task_stream my_task_stream[num_priority_levels];
-
     //! Highest priority level containing enqueued tasks
     /** It being greater than 0 means that high priority enqueued tasks had to be
         bypassed because all workers were blocked in nested dispatch loops and
         were unable to progress at then current priority level. **/
     tbb::atomic<intptr_t> my_skipped_fifo_priority;
-#else /* !__TBB_TASK_PRIORITY */
-
-    //! Task pool for the tasks scheduled via task::enqueue() method
-    /** Such scheduling guarantees eventual execution even if
-        - new tasks are constantly coming (by extracting scheduled tasks in 
-          relaxed FIFO order);
-        - the enqueuing thread does not call any of wait_for_all methods. **/
-    task_stream my_task_stream;
 #endif /* !__TBB_TASK_PRIORITY */
 
-#if __TBB_SCHEDULER_OBSERVER
-    //! List of local observers attached to this arena.
-    observer_list my_observers;
-#endif /* __TBB_SCHEDULER_OBSERVER */
+    // Below are rarely modified members
+
+    //! Market owning this arena
+    market* my_market;
+
+    //! ABA prevention marker
+    uintptr_t my_aba_epoch;
+
+#if !__TBB_FP_CONTEXT
+    //! FPU control settings of arena's master thread captured at the moment of arena instantiation.
+    cpu_ctl_env my_cpu_ctl_env;
+#endif
+
+#if __TBB_TASK_GROUP_CONTEXT
+    //! Default task group context.
+    /** Used by root tasks allocated directly by the master thread (not from inside
+        a TBB task) without explicit context specification. **/
+    task_group_context* my_default_ctx;
+#endif /* __TBB_TASK_GROUP_CONTEXT */
+
+    //! Number of slots in the arena
+    unsigned my_num_slots;
 
     //! Indicates if there is an oversubscribing worker created to service enqueued tasks.
     bool my_mandatory_concurrency;
@@ -189,14 +170,13 @@ private:
     friend class governor;
     friend class task_scheduler_observer_v3;
     friend class market;
+    friend class tbb::task;
     friend class tbb::task_group_context;
     friend class allocate_root_with_context_proxy;
     friend class intrusive_list<arena>;
-#if __TBB_TASK_ARENA
-    friend class tbb::interface6::task_arena; // included through in scheduler_common.h
-    friend class interface6::wait_task;
-    friend struct interface6::wait_body;
-#endif //__TBB_TASK_ARENA
+    friend class interface7::internal::task_arena_base; // declared in scheduler_common.h
+    friend class interface7::internal::delegated_task;
+    friend class interface7::internal::wait_task;
 
     typedef padded<arena_base> base_type;
 
@@ -259,11 +239,7 @@ private:
     bool is_out_of_work();
 
     //! enqueue a task into starvation-resistance queue
-#if __TBB_TASK_PRIORITY
-    void enqueue_task( task&, priority_t, unsigned & );
-#else /* !__TBB_TASK_PRIORITY */
-    void enqueue_task( task&, unsigned & );
-#endif /* !__TBB_TASK_PRIORITY */
+    void enqueue_task( task&, intptr_t, FastRandom & );
 
     //! Registers the worker with the arena and enters TBB scheduler dispatch loop
     void process( generic_scheduler& );
@@ -280,7 +256,10 @@ private:
 #if __TBB_TASK_PRIORITY
     //! Check if recent priority changes may bring some tasks to the current priority level soon
     /** /param tasks_present indicates presence of tasks at any priority level. **/
-    inline bool may_have_tasks ( generic_scheduler*, arena_slot&, bool& tasks_present, bool& dequeuing_possible );
+    inline bool may_have_tasks ( generic_scheduler*, bool& tasks_present, bool& dequeuing_possible );
+
+    //! Puts offloaded tasks into global list of orphaned tasks
+    void orphan_offloaded_tasks ( generic_scheduler& s );
 #endif /* __TBB_TASK_PRIORITY */
 
 #if __TBB_COUNT_TASK_NODES
@@ -292,15 +271,6 @@ private:
     arena_slot my_slots[1];
 }; // class arena
 
-} // namespace internal
-} // namespace tbb
-
-#include "market.h"
-#include "scheduler_common.h"
-#include "governor.h"
-
-namespace tbb {
-namespace internal {
 
 template<bool is_master>
 inline void arena::on_thread_leaving ( ) {
