@@ -286,6 +286,7 @@ struct _VteTerminalPrivate {
         VteCharacterReplacement *character_replacement;     /* pointer to the active one */
 
 	/* Selection information. */
+        char *word_char_exceptions;
 	gboolean has_selection;
 	gboolean selecting;
 	gboolean selecting_after_threshold;
@@ -420,6 +421,11 @@ struct _VteTerminalPrivate {
 	gboolean cursor_moved_pending;
 	gboolean contents_changed_pending;
 
+	/* desktop notification */
+	gboolean notification_received;
+	gchar *notification_summary;
+	gchar *notification_body;
+
 	/* window name changes */
         gchar *window_title;
 	gchar *window_title_changed;
@@ -474,6 +480,7 @@ void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
 void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
 void _vte_terminal_emit_text_inserted(VteTerminal *terminal);
 void _vte_terminal_cursor_down (VteTerminal *terminal);
+void _vte_terminal_drop_scrollback (VteTerminal *terminal);
 void _vte_terminal_restore_cursor (VteTerminal *terminal, VteScreen *screen);
 void _vte_terminal_save_cursor (VteTerminal *terminal, VteScreen *screen);
 gboolean _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
@@ -519,7 +526,7 @@ gboolean _vte_terminal_size_to_grid_size(VteTerminal *terminal,
                                          long *cols,
                                          long *rows);
 
-gboolean _vte_is_word_char(gunichar c) G_GNUC_CONST;
+gboolean _vte_terminal_is_word_char(VteTerminal *terminal, gunichar c);
 
 G_END_DECLS
 
